package Amethyst::Brain::Infobot::Module::Karma;

use strict;
use vars qw(@ISA);
use Amethyst::Message;
use Amethyst::Store;
use Amethyst::Brain::Infobot;
use Amethyst::Brain::Infobot::Module;

@ISA = qw(Amethyst::Brain::Infobot::Module);

sub new {
	my $class = shift;
	my $self = $class->SUPER::new(
					Name		=> 'Karma',
					Regex		=> qr/(?:(?:explain|random )?karma|\+\+|--)/i,
					Usage		=> '<foo>++|<foo>--|karma <foo>|explain karma <foo>|random karma',
					Description	=> "Karma handler",
					@_
						);

	$self->{Store} = new Amethyst::Store(
					Source	=> 'karma',
						);

	return bless $self, $class;
}

sub karma_data {
	my ($self, $term, $creat) = @_;
	my $data = $self->{Store}->get($term);
	return $data if ref $data;			# Exists and upgraded
	return [ $data, [], [] ] if $creat;	# Might exist. Not upgraded
	return undef unless $data;			# Not exist or zero: no loss
	return [ $data, [], [] ];			# Exists, not upgraded
}

sub karma_random {
	my ($self) = @_;
	my $store = $self->{Store};
	my @keys = $store->keys;
	my $term = $keys[int(rand(@keys))];
	return $term;
}

sub karma_get {
	my ($self, $term) = @_;
	my $data = $self->karma_data($term);
	return $data ? $data->[0] : 0;
}

# These two can warn about undef
sub karma_inc {
	my ($self, $term, $reason) = @_;
	my $data = $self->karma_data($term);
	if (ref($data)) {
		$data->[0]++;
	}
	else {
		$data = [ 1, [], [] ];
	}
	unshift(@{$data->[1]}, $reason) if $reason;
	$self->{Store}->set($term, $data);
}

sub karma_dec {
	my ($self, $term, $reason) = @_;
	my $data = $self->karma_data($term);
	if (ref($data)) {
		$data->[0]--;
	}
	else {
		$data = [ -1, [], [] ];
	}
	unshift(@{$data->[2]}, $reason) if $reason;
	$self->{Store}->set($term, $data);
}

sub karma_explain {
	my ($self, $term) = @_;

	my $data = $self->karma_data($term, undef);

	return "Nothing is known about $term." unless ref $data;

	my @pos = @{ $data->[1] };
	@pos = map { "\"$_\"" } grep { $_ } @pos[0..4];
	my $pos = @pos ? join(', ', @pos) : "(nothing)";

	my @neg = @{ $data->[2] };
	@neg = map { "\"$_\"" } grep { $_ } @neg[0..4];
	my $neg = @neg ? join(', ', @neg) : "(nothing)";

	return
			"Positive: $pos ... " .
			"Negative: $neg ... " .
			"Overall: $data->[0].";
}

sub action {
    my ($self, $message) = @_;

	my $content = $message->content;

	$content =~ s/\s+/ /g;
	$content =~ s/^ //;

	# my $re = qr/(\(.*?\)|[^(++)(--)\s]+)(\+\+|--)/;

	if ($content =~ /^karma (.*)/) {
		my $term = lc $1;
		$term =~ s/^[ (]*//g;
		$term =~ s/[ )]*$//g;

		my $karma = $self->karma_get($term);

		my $reply = $self->reply_to($message, "$term has " .
						($karma ? "a karma of $karma" : "no karma"));
		$reply->send;

		return 1;
	}
	elsif ($content =~ /^explain karma (.*)/) {
		my $term = lc $1;
		$term =~ s/^[ (]*//g;
		$term =~ s/[ )]*$//g;

		my $reply = $self->reply_to($message,
						$self->karma_explain($term));
		$reply->send;
		return 1;
	}
	elsif ($content =~ /^random karma/) {
		my $term = $self->karma_random;
		my $reply = $self->reply_to($message,
						"Karma for $term: " .
						$self->karma_explain($term));
		$reply->send;
		return 1;
	}
	else {
		my $retval = undef;

		my %seen = ();

		while ($content =~
				s/(\(.*?\)|[^(++)(--)\s]+)(\+\+|--)(?:\s*#\s*(.*))?//) {
			my ($term, $inc, $reason) = (lc $1, $2, $3);

			next unless $term =~ /\w/;

			$retval = 1;

			$term =~ s/^[ (]*//g;
			$term =~ s/[ )]*$//g;

			next if $seen{$term};
			$seen{$term} = 1;

			# Require public

			if ($message->channel eq CHAN_PRIVATE) {
				my $reply = $self->reply_to($message, "Karma must " .
								"be altered in public.");
				$reply->channel(CHAN_PRIVATE);
				$reply->send;
				return 1;
			}
			elsif ($term eq lc $message->user) {
				my $reply = $self->reply_to($message, "You cannot " .
								"alter your own karma");
				$reply->channel(CHAN_PRIVATE);
				$reply->send;
			}
			elsif ($inc eq '++') {
				$self->karma_inc($term, $reason);
			}
			else {
				$self->karma_dec($term, $reason);
			}
		}

		return $retval;
	}
}

1;
